function hOut = latext(varargin)
% LATEXT creates text and/or sets string, and simultaneously sets LaTeX alternative
% 
% LATEXT(X, Y, STR, ALT, ...)  % where X and Y are coordinates
% 	creates text at (X, Y), sets string STR and LaTeX alternative ALT.
% LATEXT(H, STR, ALT, ...)     % where H is a TEXT object handle.
% 	sets string STR and LaTeX alternative ALT for object H.
% LATEXT(OPT, STR, ALT, ...)
% 	set string STR and LaTeX alternative ALT for the object on the current
% 	axes indicated by OPT, where OPT is one of:
% 		'-title'
% 		'-xlabel'
% 		'-ylabel'
% 		'-zlabel'
% 		'-xtick'
% 		'-ytick'
% 		'-ztick'
% In all cases optional additional arguments (...) set object properties.
% 
% STRS = LATEXT(H)
% STRS = LATEXT(OPT)
% 	returns a cell array containing the string and LaTeX alternative for
% 	the specified object.
% 		
% H = LATEXT(....)
% 	returns TEXT object handle (or AXES object handle if it was ticks that
% 	were manipulated).

% Part of PFCMP version 2.5.41, an extension to the PSIGNIFIT toolbox for MATLAB versions 5 and up.
% The license and NO WARRANTY statement for PSIGNIFIT also applies to this release - see psych_legal.m
% Copyright (c) J.Hill 2002.
% mailto:psignifit@bootstrap-software.org
% http://www.bootstrap-software.org/psignifit/


if isnumeric(varargin{1})
	if length(varargin) >= 2, arg2 = varargin{2}; else arg2 = ''; end
	if isnumeric(arg2)
		if length(varargin) < 3, error('must supply string'), end
		txt = varargin{3};
		if length(varargin) >= 4, alt = varargin{4}; else alt = []; end
		h = text(varargin{[1:3 5:end]});
	else
		h = varargin{1};
		if length(varargin) >= 2, txt = varargin{2}; else txt = []; end
		if length(varargin) >= 3, alt = varargin{3}; else alt = []; end
		if ~isnumeric(txt), set(h, 'string', txt), end
		if length(varargin) >= 4, set(h, varargin{4:end}), end
	end
	if ~isnumeric(alt), latex(h, alt); end
else
	opt = lower(varargin{1});
	if strncmp(opt, '-', 1), opt(1) = []; end
	[opt ans err] = jmatchstr(opt, {'xlabel', 'ylabel', 'zlabel', 'title', 'xticklabel', 'yticklabel', 'zticklabel'});
	if ~isempty(err), err(1:min(findstr(err, endl))-1) = []; error(['please start with X, Y coords or one of the following options' err]), end
	if length(varargin) >= 2, txt = varargin{2}; else txt = []; end
	if length(varargin) >= 3, alt = varargin{3}; else alt = []; end
	props = varargin(4:end);
	field = 'string';
	if rem(length(props), 2), error('additional arguments must come in key/value pairs'), end
	switch opt
		case 'xlabel', h = get(gca, 'xlabel'); if ~isnumeric(txt), xlabel(txt); end
		case 'ylabel', h = get(gca, 'ylabel'); if ~isnumeric(txt), ylabel(txt); end
		case 'zlabel', h = get(gca, 'zlabel'); if ~isnumeric(txt), zlabel(txt); end
		case 'title', h = get(gca, 'title'); if ~isnumeric(txt), title(txt); end
		case 'xticklabel', h = gca; field = 'xticklabel'; if ~isnumeric(txt), xtick(gca, txt); end
		case 'yticklabel', h = gca; field = 'yticklabel'; if ~isnumeric(txt), ytick(gca, txt); end
		case 'zticklabel', h = gca; field = 'yticklabel'; if ~isnumeric(txt), ztick(gca, txt); end
	end
	if ~isempty(props), set(h, props{:}); end
	if ~isnumeric(alt), latex(h, field, alt); end
end
if isnumeric(txt)
	alt = latex(h);
	if strcmp(lower(get(h, 'type')), 'axes')
		hOut = {h, alt};
	else
		if isfield(alt, 'string'), alt = alt.string; else alt = []; end
		hOut = {get(h, 'string'), alt};
	end
elseif nargout
	hOut = h;
end
